@@ -40,9 +40,9 @@ void emu_init()
 
 void emu_reset()
 {
+	hw_reset();
 	lcd_reset();
 	cpu_reset();
-	hw_reset();
 	mbc_reset();
 	sound_reset();
 }
@@ -68,6 +68,8 @@ void emu_run()
 	void *timer = sys_timer();
 	int delay;
 
+	vid_begin();
+	lcd_begin();
 	for (;;)
 	{
 		cpu_emulate(2280);
@@ -75,13 +77,14 @@ void emu_run()
 			emu_step();
 		
 		vid_end();
+		rtc_tick();
 		sound_mix();
 		delay = pcm_submit() ? 0 : framelen - sys_elapsed(timer);
 		/* printf("%d\n", delay); */
 		sys_sleep(delay);
 		sys_elapsed(timer);
-		vid_begin();
 		doevents();
+		vid_begin();
 		if (framecount) { if (!--framecount) die("finished\n"); }
 		
 		if (!(R_LCDC & 0x80))
