@@ -25,10 +25,10 @@
 
 struct hw
 {
-	byte regs[0x100];
 	byte ilines;
 	byte pad;
-	int cgb;	
+	int cgb;
+	int hdma;
 };
 
 
