@@ -1,10 +1,25 @@
 
 
 #include <stdlib.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+
+#ifdef IS_FBSD
+#include "machine/soundcard.h"
+#define DSP_DEVICE "/dev/dsp"
+#endif
+
+#ifdef IS_OBSD
+#include "soundcard.h"
+#define DSP_DEVICE "/dev/sound"
+#endif
+
+#ifdef IS_LINUX
 #include <sys/soundcard.h>
+#define DSP_DEVICE "/dev/dsp"
+#endif
 
 #include "defs.h"
 #include "pcm.h"
@@ -16,13 +31,17 @@
 struct pcm pcm;
 
 static int dsp;
+static char *dsp_device;
+static int stereo = 1;
 static int samplerate = 44100;
 static int sound = 1;
 
 rcvar_t pcm_exports[] =
 {
 	RCV_BOOL("sound", &sound),
+	RCV_INT("stereo", &stereo),
 	RCV_INT("samplerate", &samplerate),
+	RCV_STRING("oss_device", &dsp_device),
 	RCV_END
 };
 
@@ -40,16 +59,17 @@ void pcm_init()
 		dsp = -1;
 		return;
 	}
-	
-	dsp = open("/dev/dsp", O_WRONLY);
 
-	//n = 0x10000a;
-	n = 0x80008;
+	if (!dsp_device) dsp_device = strdup(DSP_DEVICE);
+	dsp = open(dsp_device, O_WRONLY);
+
+	n = 0x80009;
 	ioctl(dsp, SNDCTL_DSP_SETFRAGMENT, &n);
 	n = AFMT_U8;
 	ioctl(dsp, SNDCTL_DSP_SETFMT, &n);
-	n = 1; 
+	n = stereo;
 	ioctl(dsp, SNDCTL_DSP_STEREO, &n);
+	pcm.stereo = n;
 	n = samplerate;
 	ioctl(dsp, SNDCTL_DSP_SPEED, &n);
 	pcm.hz = n;
@@ -57,7 +77,7 @@ void pcm_init()
 	pcm.buf = malloc(pcm.len);
 }
 
-void pcm_shutdown()
+void pcm_close()
 {
 	if (pcm.buf) free(pcm.buf);
 	memset(&pcm, 0, sizeof pcm);
